@@ -8,6 +8,7 @@ from tinymlgen import port
 from functools import reduce
 from eloquentarduino.utils import jinja
 from eloquentarduino.ml.classification.tensorflow.Layer import Layer, layers
+from eloquentarduino.ml.classification.device import ClassifierResources
 
 
 class NeuralNetwork:
@@ -38,6 +39,15 @@ class NeuralNetwork:
     def num_classes(self):
         return self.y.shape[1]
 
+    @property
+    def train_accuracy(self):
+        """
+        Get train accuracy
+        """
+        accuracy = self.history.history.get('val_accuracy', self.history.history['accuracy'])
+
+        return accuracy[-1]
+
     def clone(self):
         """
         Clone current network
@@ -154,7 +164,7 @@ class NeuralNetwork:
         """
         self.fit_options[key] = value
 
-    def fit(self, X, y):
+    def fit(self, X, y, **kwargs):
         """
         Fit the network
         :param X:
@@ -163,6 +173,8 @@ class NeuralNetwork:
         self.sequential = tf.keras.Sequential()
         y = self.to_categorical(y)
 
+        self.fit_options.update(kwargs)
+
         # build the network
         for i, layer_definition in enumerate(self.layer_definitions):
             kwargs = layer_definition.kwargs
@@ -187,6 +199,8 @@ class NeuralNetwork:
         self.X = X
         self.y = y
 
+        return self
+
     def predict(self, X):
         """
         Predict
@@ -263,6 +277,13 @@ class NeuralNetwork:
             'classmap': classmap
         })
 
+    def on_device(self, project=None):
+        """
+        Get device benchmarker
+        :param project: Project
+        """
+        return ClassifierResources(self, project=project)
+
     def to_categorical(self, y):
         """
         One-hot encode y array
